#!/bin/sh
# chkconfig: 345 77 15
# description: Plesk Interface and Utilities

# Plesk source function library
#
# PROVIDE: psa
# REQUIRE: NETWORKING SERVERS named mysql apache2 postgresql mailman
# BEFORE: DAEMON
# KEYWORD: shutdown
#
# Add the following line to /etc/rc.conf to enable PSA:
#
#  psa_enable="YES"
#
# This scripts takes one of the following commands:
#
#   start stop restart reload status 
#

. //usr/local/etc/rc.d/functions

product="psa"

admin_passwd="`cat /etc/$product/.$product.shadow`"
mysql_service="mysql-server"
MYSQL_BIN_D="/usr/local/bin"

PRODUCT_NAME="Plesk"
PRODUCT_ROOT_D="/usr/local/psa"
PRODUCT_RC_D="/usr/local/etc/rc.d"
PRODUCT_UTILS_DIR="${PRODUCT_ROOT_D}/admin/sbin"
SYSTEM_RC_D="/etc/rc.d"
OSVERSION=6.1

#set  openssl config
OPENSSL_CONF="/usr/local/psa/admin/conf/openssl.cnf"
export OPENSSL_CONF

name="psa"
_pidprefix="/var/run/psa"
pidfile="${_pidprefix}.pid"

. /etc/rc.subr

load_rc_config ${name}

rcvar=`set_rcvar`

#Set defaults
psa_enable=${psa_enable:-"NO"}
psa_flags=${psa_flags:-""}

#expand commands list
extra_commands="start1 stop1 restart1"

#Common commands
start_cmd="psa_common start"
stop_cmd="psa_common stop"
restart_cmd="psa_common restart"

#Commands for admin apache only
start1_cmd="psa_common start 1"
stop1_cmd="psa_common stop 1"
restart1_cmd="psa_common restart 1"

#Get comment string for services
inten=""

#start|stop sequence of services. 
services_list="named mysqld admin mailer postgres suexec tomcat httpd spamassassin drwebd mailman"
services_list1="admin"

psa_common()
{
	local action="$1"
	local addition="$2"
	local services

	case $addition in
	    1) services="$services_list1" ;;
	    *) services="$services_list" ;;
	esac
	case $action in
	    stop) services=`reverse_list $services` ;;
	    *) ;;
	esac

	for service in $services; do
		echo_action "$service" "$action"

		$service status
		status="$?"

		# status values:
		# 0 - service is running
		# 1 - service is stopped
		# 2 - service is not installed
		# 3 - service status is undefined and
		#        action must be executed anyway

		if [ "X$status" = "X2" ]; then
			echo "not installed"
			continue;
		fi 

		case $action in
			start*)
				if [ "X$status" = "X0" ]; then 
					echo "already started"
				else
					$service start && suc || fail
				fi
			;;
			stop*)
				if [ "X$status" = "X1" ]; then 
					 echo "already stopped"
				else
					$service stop && suc || fail 
				fi
			;;
			restart*)
				if [ "X$status" != "X1" ]; then
					$service stop || fail  
				fi 
				if [ $? -ne 1 ]; then
					$service start && suc || fail
				fi
			;;
		esac
	done 

	return 0
}

named()
{
	local action="$1"
	local service

	inten="DNS server (named)"

	if [ -x "${SYSTEM_RC_D}/named" ]; then
		service="${SYSTEM_RC_D}/named"
	else
		return 2
	fi

	case $action in
		comment) ;;
		status)
			status named > /dev/null 2>&1 || return 1
		;;
		*)
			$service $action >/dev/null 2>&1 || return 1
		;;
	esac

	return 0
}

mysqld()
{
	local action="$1"
	local service
	local flag=0

	inten="MySQL server"

	if [ -x "${PRODUCT_RC_D}/$mysql_service" ]; then
		service="${PRODUCT_RC_D}/$mysql_service"
	elif [ -x "${PRODUCT_RC_D}/${mysql_service}.sh" ]; then
		service="${PRODUCT_RC_D}/${mysql_service}.sh"
	else
		return 2
	fi

	case $action in
		comment) ;;
		start)
			$service $action >/dev/null 2>&1 || flag=1
			while ! $service status /dev/null 2>&1; do
			sleep 1
			    count=$(($count+1))
			    if test $count -gt 20; then
				return 1
			    fi
			done


			if [ "X$admin_passwd" != "X" ]; then
				query="delete from sessions"
				echo "$query" | "$MYSQL_BIN_D/mysql" -u admin -p"$admin_passwd" $product
				rm -f $PRODUCT_ROOT_D/admin/sessions/*
			fi
		;;
                status)
                        $service $action >/dev/null 2>&1
                        if [ $? -eq 1 ]; then
			    ps ax | grep -q mysql
			    if [ $? -eq 0 ]; then
				flag=1
				for i in `jot 5`; do
				    sleep 1
				    $service $action >/dev/null 2>&1 && flag=0 && break
				done
			    else
				flag=1
			    fi
                        fi
                ;;
		*)
			$service $action >/dev/null 2>&1 || flag=1
		;;
	esac

	if [ 0$flag -eq 1 ]; then
		return 1
	fi

	return 0
}

admin()
{
	local action="$1"
	local service
	local flag=0

	inten="admin server"	

	if [ -x /usr/local/etc/rc.d/sw-cp-server ]; then
		service="/usr/local/etc/rc.d/sw-cp-server"
	else
		return 2
	fi

	case "$action" in
		comment) ;;
		start)
			$service start > /dev/null 2>&1 || flag=1
			$PRODUCT_ROOT_D/admin/bin/packagemng --set-dirty-flag
		;;
		stop)
			$service stop > /dev/null 2>&1 || flag=1 
		;;
		status)
			$service status > /dev/null 2>&1 || flag=1
		;;
	esac

	if [ 0$flag -eq 1 ]; then
		return 1
	fi
 
	return 0
}

mailer()
{
	local action="$1"
	local service
	local driver
	local subservno
	local subservrun

	inten="Mail Server"

	driver="$PRODUCT_UTILS_DIR/mailmng"

	if [ ! -x $driver ]; then
		return 2
	fi

	case $action in
		comment) ;;
		start)
			$driver --start-service >/dev/null 2>&1 || return 1
		;;
		stop)
			$driver --stop-service >/dev/null 2>&1 || return 1
		;;
		status)
			# get total number of mailer services
			subservno=`$driver --status-service | wc -l`
			# get number of running services
			subservrun=`$driver --status-service | grep 'is running' | wc -l`
			if [ $subservrun -eq 0 ]; then
				return 1
			fi
			if [ $subservno -ne $subservrun ]; then
				# some mail services are running and some not
				return 3
			fi
		;;
		restart)
			$driver --restart-service  >/dev/null 2>&1 || return 1
		;;
		reload)
			$driver --reload-service  >/dev/null 2>&1 || return 1
		;;
	esac

	return 0
}

postgres()
{
	local action="$1"
	local service

	inten="PostgreSQL Server"

	if [ -x "$PRODUCT_RC_D/postgresql" ]; then
		service="$PRODUCT_RC_D/postgresql"
	elif [ -x "$PRODUCT_RC_D/postgresql.sh" ]; then
		service="$PRODUCT_RC_D/postgresql.sh"
	else 
		return 2
	fi

	case $action in
		comment) ;;
		*)
			$service $action >/dev/null 2>&1 || return 1
		;;
	esac

	return 0
}

suexec()
{
	local action="$1"
	local service
	
	inten="Register suexec utility"

	if [ -x "$PRODUCT_RC_D/psa-suexec.sh" ]; then
		service="$PRODUCT_RC_D/psa-suexec.sh"   
	else
		return 2
	fi

	case "$action" in
		comment) ;;
		stop) return 0 ;;
		*) $service $action >/dev/null 2>&1 || return 1 ;;
	esac
}

httpd()
{

	local action="$1"
	local service

	inten="Apache Server"
	
	is_apache20="`pkg_info -qO www/apache20`"
	is_apache22="`pkg_info -qO www/apache22`"

	if [ ! -z "$is_apache20" ]; then
	    apache_suffix="apache2"
	elif [ ! -z "$is_apache22" ]; then
	    apache_suffix="apache22"
	else
	    return 2
	fi

	if [ -x "$PRODUCT_RC_D/$apache_suffix" ]; then
		service="$PRODUCT_RC_D/$apache_suffix"   
	elif [ -x "$PRODUCT_RC_D/${apache_suffix}.sh" ]; then
		service="$PRODUCT_RC_D/${apache_suffix}.sh"   
	else
		return 2
	fi
	
	case "$action" in
		comment) ;;
		start)
			${PRODUCT_ROOT_D}/admin/sbin/websrvmng -p -v >/dev/null || return 1
		;;
		stop)
			$service stop >/dev/null 2>&1 || return 1
		;;
		status)
			status="`${PRODUCT_ROOT_D}/admin/sbin/websrvmng -s`"
			[ "X$status" = "Xis stopped" ] && return 1 || return 0
		;;
	esac

	return 0
}

tomcat()
{
	local action="$1"
	local service

	inten="Tomcat server"

	if [ -x "$PRODUCT_RC_D/tomcat4" ]; then
	    service="$PRODUCT_RC_D/tomcat4"
	elif [ -x "$PRODUCT_RC_D/tomcat5" ]; then
	    service="$PRODUCT_RC_D/tomcat5"
	else
	    return 2
	fi

	case "$action" in
	    comment) ;;
	    status)
		status="`${PRODUCT_ROOT_D}/admin/sbin/tomcatmng -s 2>/dev/null`"
		[ "X$status" = "Xis stopped" ] && return 1 || return 0
	    ;;
	    *)
		$service $action >/dev/null 2>&1 || return 1
	    ;;
	esac

	return 0
}

spamassassin()
{
	local action="$1"
	local service

	inten="Spamassassin daemon"

	if [ -x "$PRODUCT_RC_D/psa-spamassassin" ]; then
		service="$PRODUCT_RC_D/psa-spamassassin"
	else
		return 2
	fi

	case $action in
		comment) ;;
		status)
			status="`$service status`"
			[ "X$status" = "Xis running" ] && return 0 || return 1
		;;
		*) 
			$service $action >/dev/null 2>&1 || return 1
		;;
	esac
}

drwebd()
{
	local action="$1"
	local service 

	inten="DrWeb antivirus daemon"

    if [ -x "$PRODUCT_RC_D/drwebd" ]; then
		service="$PRODUCT_RC_D/drwebd"
    elif [ -x "$PRODUCT_RC_D/00.drwebd.sh" ]; then
		service="$PRODUCT_RC_D/00.drwebd.sh"
	else 
		return 2
	fi

	case $action in
		comment) ;;
		status)
			status drwebd >/dev/null || return 1
		;;
		*) 
			$service $action >/dev/null 2>&1 || return 1
		;;
	esac

	return 0
}

mailman()
{
	local action="$1"
	local service
	
	inten="Mailman Server"

    if [ -x "$PRODUCT_RC_D/mailman" ]; then
		service="$PRODUCT_RC_D/mailman"
	else
		return 2
	fi

	case $action in
		comment) ;;
		*)
	    	$service $action >/dev/null 2>&1 || return 1
		;;
	esac

	return 0
}

echo_action()
{
	local service="$1"
	local action="$2"

	case $action in
		start) msg="Starting" ;;
		stop) msg="Stopping" ;;
		restart) msg="Restarting" ;;
		*) ;;
	esac
	
	$service comment 

	echo -n "Plesk: $msg $inten... "
	return 0
}

suc()
{
	echo "done"
	return 0
}

fail()
{
	echo "failed"
	return 1
}

run_rc_command "$1"

exit 0

